<div class="form-group">
  <label for="<%= name %>"><%= name.capitalize %></label>
  <textarea class="form-control" rows="5" id="<%= name %>" name="<%= name %>" <%= "disabled" if disabled %>><%= value %></textarea>
</div>
