<div class="form-group">
  <label for="<%= name %>"><%= name.capitalize %></label>
  <input type="text" class="form-control" id="<%= name %>" name="<%= name %>" value="<%= value %>" <%= "disabled" if disabled %>>
</div>
