<div class="form-group">
  <label for="<%= name %>"><%= name.capitalize %></label>
  <select class="form-control" id="<%= name %>" name="<%= name %>" <%= "disabled" if disabled %>>
    <option value="">(none)</option>
    <% select_fields[name].each do |v| %>
      <option <%= "selected" if (v == value) %>><%= v %></option>
    <% end %>
  </select>
</div>
