<h1>Search Results: <%= @query %></h1>
<ul class='list-group' style='max-width: 400px'>
  <% results.records.each do |package| %>
  <li class='list-group-item'>
  <a href='/packages/<%= package.prefix %>/<%= package.name %>'>
      <%= package.name %>
    </a></li>
  <% end %>
</ul>
<%== erb(:pagination, locals: { results: results }) %>
