<div class="checkbox">
  <label for="<%= name %>">
    <input type="hidden" name="<%= name %>" value="0"/>
    <input type="checkbox" id="<%= name %>" name="<%= name %>" value="1" <%= "checked" if value %> <%= "disabled" if disabled %>/>
    <%= name.capitalize %>
  </label>
</div>
