# shellcheck disable=SC2154
incus_or_lxd="$(echo "${debci_backend}" | sed -e 's/-.*//')"
incus_or_lxd_vm=
case "${debci_backend}" in
  *-vm)
    incus_or_lxd_vm=--vm
    ;;
esac
export incus_or_lxd
export incus_or_lxd_vm

debci_incus_tmpdir=$(mktemp -d -t debci-incus-XXXXXXXXXXXXXXX)
trap 'rm -rf ${debci_incus_tmpdir}' INT TERM EXIT
export INCUS_CONF="${debci_incus_tmpdir}/config"

debci_incus_image="autopkgtest/${debci_distro:?}/${debci_suite:?}/${debci_arch:?}"
