<div class='row'>
  <div class='col-lg-12'>
    <nav aria-label="breadcrumb">
      <ol class='breadcrumb'>
        <li class="breadcrumb-item"><a href='/packages/<%= @package.prefix %>/'><%= @package.prefix %></a></li>
        <li class="breadcrumb-item"><a href='/packages/<%= @package.prefix %>/<%= @package.name %>'><%= @package.name %></a></li>
        <li class="breadcrumb-item active"><%= @suite %>/<%= @architecture %></li>
      </ol>
    </nav>

    <h2>
      <%= @package.name %>
      <small>
        [ <%= @suite %>/<%= @architecture %> ]
      </small>
      <%== erb(:package_links) %>
    </h2>

    <div class='job-list-legend'>
      Legend:
      <span class='pinned-false'>Test with packages from <em><%= @suite %></em> only</span>
      <span class='pinned-true'>Test with packages from other suites</span>
    </div>

    <% if @package.reject_listed? %>
      <div class='alert alert-warning'>
        This package is currently <a href="/status/reject_list/">reject_listed</a> and will not have any new test runs.
      </div>
    <% end %>

 <%== erb(:job_listing, locals: { jobs_list: results.records, fields: [:version, :date, :trigger, :backend, :duration, :status, :requestor, :results ]}) %>
 <%== erb(:pagination, locals: { results: results }) %>

  </div>
</div>
