<table class="table">
  <% prefix = job.package.prefix %>
  <tbody>

    <tr>
      <td>Package name:</td>
      <td><a href="/packages/<%= prefix %>/<%= job.package.name %>/"><%= job.package.name %></a></td>
    </tr>
    <tr>
      <td>Suite and Arch:</td>
      <td>
        <a href="<%= '/packages/%s/%s/%s/%s/' % [prefix, job.package.name, job.suite, job.arch] %>">
          <%= job.suite %>/<%= job.arch %>
        </a>
      </td>
    </tr>
    <tr>
      <td>Trigger:</td>
      <td>
        <% if job.trigger.nil? %>
          &mdash;
        <% else %>
          <%= job.trigger %>
        <% end %>
      </td>
    </tr>
    <tr>
      <td>Pinned packages:&nbsp;</td>
      <td>
        <% if job.pin_packages.empty? %>
          &mdash;
        <% else %>
          <% job.pin_packages.each do |package| %>
            <p>
              <% prefix = job.package.prefix %>
              <a href="/packages/<%= prefix %>/<%= package %>/">
                <%= package %>
              </a>
            </p>
          <% end %>
        <% end %>
      </td>
    </tr>
    <tr>
      <td>Extra APT Sources:&nbsp;</td>
      <td>
        <% if job.extra_apt_sources.empty? %>
          &mdash;
        <% else %>
          <% job.extra_apt_sources.each do |extra_apt_source| %>
            <p>
              <%= extra_apt_source %>
            </p>
          <% end %>
        <% end %>
      </td>
    </tr>

  </tbody>
</table>
