<nav aria-label="breadcrumb">
  <ol id='status-nav' class='breadcrumb'>
    <li class="breadcrumb-item"><a href='/status/'                          ><i class='fa fa-area-chart'          ></i> Status charts</a ></li>
    <li class="breadcrumb-item"><a href='/status/queue/'                    ><i class='fa fa-th'                  ></i> Queue</a ></li>
    <li class="breadcrumb-item"><a href='/status/alerts/'                   ><i class='fa fa-exclamation-triangle'></i> Status alerts</a></li>
    <li class="breadcrumb-item"><a href='/status/failing/'                  ><i class='fa fa-times'               ></i> Status failing</a></li>
    <li class="breadcrumb-item"><a href='/status/pending/'                  ><i class='fa fa-question-circle'     ></i> Pending jobs</a></li>
    <li class="breadcrumb-item"><a href='/status/reject_list/'              ><i class='fa fa-hand-paper-o'        ></i> RejectList</a></li>
    <li class="breadcrumb-item"><a href='/status/platform-specific-issues/' ><i class='fa fa-eye'                 ></i> Platform-specific issues</a></li>
    <li class="breadcrumb-item"><a href='/status/slow/'                     ><i class='fa fa-clock-o'             ></i> Slow running tests</a></li>
  </ol>
</nav>
