<div class="card">
  <div class="card-header card-header-browse">
    <h2>Browse packages</h2>
  </div>
  <div class="card-body">
    <ul class="package-prefix-list">
      <% @package_prefixes.each do |prefix| %>
        <li><a href="/packages/<%= prefix %>/"><%= prefix %></a></li>
      <% end %>
      <% if @package_prefixes.empty? %>
        <li><em>(no data yet)</em></li>
      <% end %>
    </ul>
  </div>
</div>
