<h1>Expired log file</h1>

<p class="alert alert-warning">
You probably followed a link to this log file, but it has been deleted due to
our data retention policy. Logs get deleted
<%= Debci.config.data_retention_days %> days after their creation, or when the
package they belong is using too much disk space.
</p>

<h2>Job info</h2>
<%== erb(:_job_info, locals: {job: @job}) %>

<h2>What to do from now</h2>
<p>
You have a few options:
</p>

<ul>
  <li>Check for newer logs in the <a href="<%= history_url(@job) %>">package history page</a>.</li>
  <li>
    <p>
    <% if logged_in? %>
      <a href="/user/<%= @user.username %>/retry/<%= @job.run_id %>" ><i class='fa fa-recycle'></i> Retry</a> the original test.
    <% else %>
      You can retry this test if you log in.
    <% end %>
    </p>
    <p>
    Note that the longer it has passed since the original test, the less likely
    it is that retrying it will reproduce the original results, or even work at
    all. This will vary on a case by case basis, based on the test parameters
    and on how much the archive has changed since the original test run.
    </p>
  </li>
</ul>
