<h1>403 Forbidden: missing authentication</h1>

<p>
To retry a test job, you need to be authenticated with a client certificate
(e.g. Debian SSO), or with a API key. See the <a href="../../doc">API
documentation</a> for more information.
</p>

<p>
Once you are properly authenticated, just reload this page to get the retry
form.
</p>
<noscript>
  <style type='text/css'>
    form { display: none }
  </style>
</noscript>
