<h1>Queue</h1>

<%== erb(:status_nav ) %>

<h2>Average Latency</h2>

<p class="alert alert-info">
Average time between submission and results being received, for jobs finished
within the last 6 hours. Updated once an hour.
</p>

<table class='table'>
  <tr>
    <th>Architecture</th>
    <th>Average latency</th>
  </tr>
  <% @latencies.each do |entry| %>
    <tr>
      <td><%= entry.arch %></td>
      <td><%= entry.in_words %></td>
    </tr>
  <% end %>
</table>

<h2>Oldest job in the queue</h2>

<p class="alert alert-info">
Considering only jobs newer than one month old. Updated once an hour.
</p>

<table class='table'>
  <tr>
    <th>Architecture</th>
    <th>Oldest job</th>
  </tr>
  <% @oldest_job.each do |entry| %>
    <tr>
      <td><%= entry.arch %></td>
      <td><%= entry.oldest_job %></td>
    </tr>
  <% end %>
</table>
