<div class="card text-bg-warning">
  <div class="card-header">
    Authentication required
  </div>
  <div class="card-body">
    <p>
    Browsing the data is only available to authenticated users.
    <a href="/user/login?original_url=<%= request.path %>">Log in</a> to
    continue.
    </p>
  </div>
</div>
