<div class="row">
  <div class="col-md-4">
    <%== erb :_browse_prefixes %>
  </div>

  <div class="col-md-8">
    <h2>
      Packages
    </h2>
    <table class="table" style="text-align: left">
      <% @packages.in_groups_of(3).each do |packages| %>
        <tr>
          <% packages.each do |package| %>
          <td>
            <% if package %>
              <a href='<%= "/packages/#{@prefix}/#{package.name}/"%>'><%= package.name %></a>
            <% end %>
          </td>
        <% end %>
        </tr>
      <% end %>
    </table>
  </div>
</div>
