<nav aria-label="breadcrumb">
  <ol class="breadcrumb">
    <li class="breadcrumb-item active">Welcome <%= @user.username %>!</a></li>
  </ol>
</nav>
<header class="well">
  <h1>Self Service</h1>
  <p>You can request a test through the debci UI or view your test request history.</p>
</header>
<div class="row">
  <div class="col-xs-12 col-md-6">
    <a class="well self-service-card text-center" href="/user/<%= @user.username %>/test">
      <div class="card">
        <div class="card-body">
          <div class="self-service-card-icon">
            <span class="fa fa-play-circle" aria-hidden="true"></span>
          </div>
          <p>Request Test</p>
        </div>
      </div>
    </a>
  </div>
  <div class="col-xs-12 col-md-6">
    <a class="well self-service-card text-center" href="/user/<%= @user.username %>/jobs">
      <div class="card">
        <div class="card-body">
          <div class="self-service-card-icon">
            <span class="fa fa-list" aria-hidden="true"></span>
          </div>
          <p>Job History</p>
        </div>
      </div>
    </a>
  </div>
  <div class="col-xs-12 col-md-6">
    <a class="well self-service-card text-center" href="/user/<%= @user.username %>/keys">
      <div class="card">
        <div class="card-body">
          <div class="self-service-card-icon">
            <span class="fa fa-lock" aria-hidden="true"></span>
          </div>
          <p>Manage API keys</p>
        </div>
      </div>
    </a>
  </div>
  <% if @user.admin %>
    <div class="col-xs-12 col-md-6">
      <a class="well self-service-card text-center" href="/admin/">
        <div class="card">
          <div class="card-body">
            <div class="self-service-card-icon">
              <span class="fa fa-cog" aria-hidden="true"></span>
            </div>
            <p>Admin Interface</p>
          </div>
        </div>
      </a>
    </div>
  <% end %>
</div>
