#status h3 {
  border-bottom: 1px solid #888a85;
}

.chart {
  height: 250px;
}

.chart h4 {
  text-align: center;
}

.chart-canvas {
  background-color: #eeeeec;
  width: 90%;
  height: 200px;
}

/* -----------------------------------------------------------------
 * headings
 * ----------------------------------------------------------------- */

.page-header {
  margin-top: 0px;
}

/* -----------------------------------------------------------------
 * table utilities
 * ----------------------------------------------------------------- */

table tr:hover td {
  background-color: var(--bs-dark-bg-subtle);
}

td a:only-child {
  display: block;
}


/* -----------------------------------------------------------------
 * status marks
 * ----------------------------------------------------------------- */

.pass {
  color: #8ae234;
}

.neutral {
  color: #8080ff;
}

.fail {
  color: #ef2929;
}

.tmpfail {
  color: #ffd862;
}

.no_test_data {
  color: #babdb6;
}

.fail_passed_never {
  color: #ef2929;
}

.fail_passed_current {
  color: #9e0000;
}

.fail_passed_old {
  color: #c80c0c;
}

.tmpfail_pass {
  color: #fffa62;
}

.tmpfail_fail {
  color: #ffb062;
}

/* -----------------------------------------------------------------
 * icons
 * ----------------------------------------------------------------- */

.fa-exclamation-triangle:before {
  color: orange;
}

.fa-rss:before {
  color: orange;
}

.fa-question-circle:before {
  color: green;
}

/* -----------------------------------------------------------------
 * navbar
 * ----------------------------------------------------------------- */

.navbar-brand {
  background: transparent url(debian.png) 0px 50% no-repeat;
  padding-left: 50px !important;
}

.navbar {
  margin-bottom: 1em;
}

/* -----------------------------------------------------------------
 * footer
 * ----------------------------------------------------------------- */
#wrap {
  min-height: 100%;
  height: auto !important;
  height: 100%;
  margin: 0 auto -60px;
  padding: 0 0 60px;
}
#footer {
  margin-top: 3em;
  padding: 1em;
  min-height: 60px;
}

#footer .row {
  margin-bottom: 1em;
}

#footer form {
  margin: 0;
  padding: 0;
  display: inline;
}

#footer button[type=submit] {
  border: none;
  background: none;
  text-decoration: underline;
  color: var(--bs-link-color);
}

/* -----------------------------------------------------------------
 * package browsing by prefix
 * ----------------------------------------------------------------- */

.package-prefix-list {
  margin: 0;
  padding: 0;
}
.package-prefix-list li {
 display: inline-block;
 margin-right: 0.25em;
}

.package-prefix-list a {
  color: #E90051;
  background-color: #e3e7ef;
  padding: 1px 10px;
  border-radius: 5px;
  text-decoration: none;
}

.package-prefix-list a:hover {
  text-decoration: none;
  background-color: #E90051;
  color: white;
}

/* -----------------------------------------------------------------
 * API doc
 * ----------------------------------------------------------------- */

#api-doc h2 a { display: none; }
#api-doc h2:hover a { display: inline; }

/* -----------------------------------------------------------------
 * Self service
 * ----------------------------------------------------------------- */

.self-service-card:link {
  display: block;
  text-decoration: none;
  margin-bottom: 1.5em;
}

.self-service-card:hover .card {
  background: var(--bs-secondary-color);
  color: var(--bs-secondary-bg);
}

.self-service-card-icon {
  display: inline-block;
  font-size: 2em;
  margin-bottom: 1em;
}

.arch-checkbox, .checkbox-inline {
  display: inline-block;
  width: 20%;
}

/* -----------------------------------------------------------------
 * Job listings
 * ----------------------------------------------------------------- */
.job-list-legend {
  margin-top: 1.5em;
  margin-bottom: 1.5em;
}
.job-list-legend span {
  display: inline-block;
  padding: 0.5em;
  margin-left: 1em;
  border-top: 1px solid;
  border-bottom: 1px solid;
}

table.job-list tr.pinned-true td {
  background-color: transparent;
}

span.pinned-true,
table.job-list tr.pinned-true td,
table.job-list tr.pinned-true td a {
  color: #888a85 !important;
}

table.job-list div.trigger-pinned-packages {
  max-width: 250px;
}


/* -----------------------------------------------------------------
 * Log viewer
 * ----------------------------------------------------------------- */

.log-actions {
 margin-bottom: 1em;
}

.log-view .log-divider {
  background-color: #2e3436;
}

.log-view .log-divider a {
  font-family: monospace;
  font-size: smaller;
  color: #66ff66;
  font-weight: bold;
  padding-left: 50px;
}

.log-view .log-divider-fail a {
  color: #ff6666;
}

.log-view .log-divider .log-open {
  display: none;
}

.log-view .line-number {
  float: left;
  width: 50px;
  font-family: monospace;
  font-size: smaller;
  text-align: right;
  color: #eeeeec;
  user-select: none;
}

.line-number a,
.line-number a:visited {
  padding-right: 0.5em;
  display: block;
  color: #eeeeec;
}
.line-number a:hover {
  background-color: #ffc;
  color: #888a85;
}

.log-view .log-line {
  margin-left: 50px;
  padding-left: 0.5em;
  padding-right: 0.5em;
  font-family: monospace;
  font-size: smaller;
  background-color: #2e3436;
  color: #eeeeec;
  white-space: pre-wrap;
}

.log-section > .line-number:first-child + .log-line,
.log-section > .line-number:first-child {
  padding-top: 0.5em;
}

.log-section .log-line:last-child {
  padding-bottom: 0.5em;
}

.log-view {
  border: 1px solid #040000;
}

.log-section {
  background-color: #555753;
}

.log-view .log-nav {
  float: right;
  margin-right: 1em;
}

#logview-index div.list-group {
  margin-bottom: 1em;
}

#logview-index a:hover {
  background-color: #ffc;
}

/* -----------------------------------------------------------------
 * Log viewer
 * ----------------------------------------------------------------- */

details code pre {
  background-color: var(--bs-dark-bg-subtle);
  margin-top: 1em;
  padding: 1em;
}
