<div class='row'>
  <div class='col-md-12'>
    <h1 class='page-header'>Status <small>/ Slow running tests</small></h1>
    <%== erb(:status_nav ) %>
    <%== erb(:job_filter ) %>

    <%== erb(:job_listing, locals: {jobs_list: @slow.records, fields: [:package, :version, :suite, :arch, :status, :duration]}) %>
    <%== erb(:pagination, locals: {results: @slow}) %>

  </div> <!-- class='col-md-12' -->
</div> <!-- class='row' -->
