require 'debci/amqp'

describe Debci::AMQP do
  it 'calculates queue name' do
    q = Debci::AMQP.queue_name('arm64', 'lxc')
    expect(q).to eq('debci-tests-arm64-lxc')
  end

  it 'aliases incus-lxc to lxc' do
    q = Debci::AMQP.queue_name('arm64', 'incus-lxc')
    expect(q).to eq('debci-tests-arm64-lxc')
  end

  it 'aliases lxd-lxc to lxc' do
    q = Debci::AMQP.queue_name('arm64', 'lxd-lxc')
    expect(q).to eq('debci-tests-arm64-lxc')
  end
end
