<div class='row'>
  <div class='col-md-12'>
    <h1 class='page-header'>Status <small>/ Pending Jobs</small></h1>
    <%== erb(:status_nav ) %>
    <%== erb(:job_filter ) %>

    <%== erb(:job_listing, locals: { jobs_list: @pending.records, fields: [:package, :suite, :arch, :requestor, :trigger, :requested]}) %>
    <%== erb(:pagination, locals: { results: @pending}) %>

  </div> <!-- class='col-md-12' -->
</div> <!-- class='row' -->

<style>
.active {
  font-weight: bold;
}
</style>
