<form>
  <div class="row">
    <div class="col-xs-6">
      <div class="form-group">
        <label for="suite">Suite</label>
        <% settings.suites.each do |suite| %>
          <label class="checkbox-inline">
            <input name="suite[]" type="checkbox" value="<%= suite %>" <%= params[:suite].is_a?(Array) && params[:suite].include?(suite) ? "checked" : nil %>><%= suite %>
          </label>
        <% end %>
      </div>
    </div>
    <div class="col-xs-6">
      <div class="form-group">
        <label for="arch">Architecture</label>
        <% settings.archs.each do |arch| %>
          <label class="checkbox-inline">
            <input name="arch[]" type="checkbox" value="<%= arch %>"<%= params[:arch].is_a?(Array) && params[:arch].include?(arch) ? "checked" : nil %>><%= arch %>
          </label>
        <% end %>
      </div>
    </div>
  </div>
  <div class="form-group">
    <button type="submit" class="btn btn-secondary" autocomplete="off">
      Apply Filters
    </button>
  </div>
</form>