<div class="row">

  <div class="col-md-4">

    <%== erb :_browse_prefixes %>

  </div> <!-- class='col-md-6' -->
  <div class="col-md-8">
    <h2>Search Package</h2>
    <form action='/packages/-/search' class="row row-cols-lg-auto g-3 align-items-center">

      <div class="col-12">
        <div class="input-group">
          <input type="text" class="form-control" id="query" name="query" size="40" placeholder="Package to search...">
        </div>
      </div>

      <div class="col-12">
        <button type="submit" class="btn btn-primary">Search</button>
      </div>
    </form>
  </div> <!-- class='col-md-6' -->

</div> <!-- class='row' -->
