class IncreaseJobVersion < ActiveRecord::Migration[4.2]
  def up
    change_column :jobs, :version, :string, limit: 256
  end

  def down
    change_column :jobs, :version, :string, limit: 100
  end
end
