<h1>Experiments</h1>

<table class='table'>
  <tr>
    <th>#</th>
    <th>Title</th>
    <th>Owner</th>
  </tr>
<% @experiments.records.each do |experiment| %>
  <tr>
    <td><a href="<%= experiment.id %>/">#<%= experiment.id %></a></td>
    <td><a href="<%= experiment.id %>/"><%= experiment.title || '(untitled)' %></a></td>
    <td><%= experiment.owner.username %></td>
  </tr>
<% end %>
</table>

<%== erb(:pagination, locals: { results: @experiments }) %>
