<div class="row">
  <div class="col-md-12">
    <h1 class='page-header'>Status <small>/ Platform-Specific issues</small></h1>
    <%== erb(:status_nav ) %>

    <table class='table table-striped'>
      <tr>
        <th>Package</th>
        <th>Status</th>
      </tr>
      <% @issues.each do |package, statuses| %>
        <tr>
          <td><%= package.name %></td>
          <td>
            <% statuses.each do |status| %>
              <a  href="/packages/<%= package.prefix %>/<%= package.name %>/<%= status.suite %>/<%= status.arch %>/">
                <%== icon(status.status) %>
                <%= status.suite %>/<%= status.arch %> <%= status.status %> (<%= status.duration_human %>)
              </a>
              &nbsp;
            <% end %>
          </td>
        </tr>
      <% end %>
    </table>
  </div>
</div>
