<div class='row'>
  <div class='col-md-12'>
    <h1 class='page-header'>Status <small>/ Alerts</small></h1>
    <%== erb(:status_nav ) %>
    <%== erb(:job_filter ) %>

    <h4>There are currently <b><%= @alert_number %></b> packages with status alerts.</h4>
    <%== erb(:job_listing, locals: { jobs_list: @tmpfail.records ,fields: [:package, :suite, :arch, :status]}) %>
    <%== erb(:pagination, locals: { results: @tmpfail }) %>

  </div> <!-- class='col-md-12' -->
</div> <!-- class='row' -->
