class CreateExperiments < ActiveRecord::Migration[7.2]
  def change
    create_table(:experiments) do |t|
      t.integer :owner_id,          null: false
      t.string :title
      t.string :suite,              null: false
      t.string :arch,               null: false
      t.integer :priority
      t.string :package_selector,   null: false
      t.string :control_parameters
      t.string :test_parameters,    null: false
      t.timestamps
      t.datetime :started_at
    end
    add_foreign_key :experiments, :users, column: :owner_id
  end
end
